@@ -3,8 +3,8 @@ import datetime as dt
 from enum import Enum
 from pyvit.can import Frame
 
-STALE_TIME = dt.timedelta(minutes=2)
-DEAD_TIME = dt.timedelta(minutes=4)
+STALE_TIME = dt.timedelta(seconds=5)
+DEAD_TIME = dt.timedelta(seconds=10)
 
 
 class MessageType(Enum):
@@ -141,7 +141,7 @@ class Message(Frame):
 
     def __init__(self: Message, arb_id: int, **kwargs):
         super().__init__(arb_id, **kwargs)
-        self.node_name = MessageType.cob_to_node(self.type, self.arb_id)
+        self.node_name = 'N/A'
         self.message = self.data
 
     @property
