@@ -1,8 +1,9 @@
+from __future__ import annotations
 import array
-import datetime
+from datetime import datetime, timedelta
 from struct import unpack
-from .eds import EDS
-from typing import List
+from typing import List, Union
+from .eds import DataType, EDS
 
 
 class FailedValidationError(Exception):
@@ -32,25 +33,27 @@ class FailedValidationError(Exception):
         self.parse_type = parse_type
         self.sub_type = sub_type
         self.message = message
-        self.time_occured = datetime.datetime.now()
+        self.time_occured = datetime.now()
         super().__init__(self.message)
 
 
-def get_name(eds_config: EDS, index: List[int]) -> (str, str):
+def get_name(eds_config: EDS, index: Union[List[int], bytes]) -> (str, str):
     """
     Get the name and data type for a given index
     :param eds_config: An EDS file for the current node
     :param index: the index and subindex to retrieve data from
                   expected to be length 3. (not validated)
-    :return: a tuple containing the name and data type as a string
+    :return: (str, str): a tuple containing the name and data type as a string
+    :raise: IndexError: The index or subindex failed to find a value in the
+    provided OD file
+    :raise: ValueError: The provided index/subindex does not contain a
+    parameter_name and data_type attribute
     """
     index_bytes = list(map(lambda x: hex(x)[2:].rjust(2, '0'), index))
     key = int('0x' + ''.join(index_bytes[:2]), 16)
     subindex_key = int('0x' + ''.join(index_bytes[2:3]), 16)
-    current = eds_config[hex(key)]
-    if current is None:
-        return "Unknown", "Unknown"
 
+    current = eds_config[hex(key)]
     result = eds_config[hex(key)].parameter_name
 
     if len(current) > 0:
@@ -62,53 +65,73 @@ def get_name(eds_config: EDS, index: List[int]) -> (str, str):
     return defined_type, result
 
 
-BOOLEAN = '0x0001'
-INTEGER8 = '0x0002'
-INTEGER16 = '0x0003'
-INTEGER32 = '0x0004'
-UNSIGNED8 = '0x0005'
-UNSIGNED16 = '0x0007'
-UNSIGNED32 = '0x0003'
-REAL32 = '0x0008'
-VISIBLE_STRING = '0x0009'
-OCTET_STRING = '0x000A'
-UNICODE_STRING = '0x000B'
-DOMAIN = '0x000F'
-REAL64 = '0x0011'
-INTEGER64 = '0x0015'
-UNSIGNED64 = '0x001B'
-
-
-def decode(defined_type: str, data: List[int]) -> str:
+def decode(defined_type: Union[str, DataType], data: List[int]) -> str:
     """
     Decodes data by defined type
     :param defined_type: Hex constant for type
     :param data: list of ints to be decoded
     :return: Decoded data as string
+    :raise: ValueError: Indicates datatype provided is not supported
     """
-    if defined_type in (UNSIGNED8, UNSIGNED16, UNSIGNED32, UNSIGNED64):
+    if defined_type in DataType.UNSIGNED_INTEGERS.value:
         result = str(int.from_bytes(data, byteorder="little", signed=False))
-    elif defined_type in (INTEGER8, INTEGER16, INTEGER32, INTEGER64):
+    elif defined_type in DataType.SIGNED_INTEGERS.value:
         result = str(int.from_bytes(data, byteorder="little", signed=True))
-    elif defined_type == BOOLEAN:
+    elif defined_type == DataType.BOOLEAN.value:
         if int.from_bytes(data, byteorder="little", signed=False) > 0:
             result = str(True)
         else:
             result = str(False)
-    elif defined_type in (REAL32, REAL64):
+    elif defined_type in DataType.FLOATING_POINTS.value:
         data = array.array('B', data).tobytes()
         result = str(unpack('>f', data)[0])
-    elif defined_type == VISIBLE_STRING:
+    elif defined_type == DataType.VISIBLE_STRING.value:
         data = array.array('B', data).tobytes()
         result = data.decode('utf-8')
-    elif defined_type in (OCTET_STRING, DOMAIN):
+    elif defined_type == DataType.OCTET_STRING.value:
         data = list(map(lambda x: hex(x)[2:].rjust(2, '0'), data))
         result = '0x' + ''.join(data)
-    elif defined_type == UNICODE_STRING:
+    elif defined_type == DataType.UNICODE_STRING.value:
         data = array.array('B', data).tobytes()
         result = data.decode('utf-16-be')
+    elif defined_type == DataType.TIME_OF_DAY.value:
+        delta = get_time_values(data)
+        date = datetime(1984, 1, 1) + delta
+        result = date.isoformat()
+    elif defined_type == DataType.TIME_DIFFERENCE.value:
+        result = str(get_time_values(data))
+    elif defined_type in DataType.NON_FORMATTED.value:
+        result = format_bytes(data)
+    elif defined_type == DataType.ECSS_TIME.value:
+        # This is ECSS SCET Time
+        # data[0:4]: Fine Time: Microseconds
+        # data[4:8]: Coarse Time: Seconds
+        coarse = data[4:8]
+        fine = int.from_bytes(data[:4], byteorder="little", signed=False)
+        coarse = int.from_bytes(data[4:8], byteorder="little", signed=False)
+        delta = timedelta(seconds=coarse, microseconds=fine)
+        date = datetime(1970, 1, 1) + delta
+        result = date.isoformat()
     else:
         raise ValueError(f"Invalid data type {defined_type}. "
                          f"Unable to decode data {str(data)}")
 
     return result
+
+
+def get_time_values(data: [int]) -> timedelta:
+    # Component ms is the time in milliseconds after midnight. Component
+    # days is the number of days since January 1, 1984.
+    # Format UNSIGNED 28 (ms), VOID4, UNSIGNED 16 (Days)
+    ms_raw = data[:4]
+    ms_raw[3] = ms_raw[3] >> 4
+    ms = int.from_bytes(ms_raw, byteorder="little", signed=False)
+    days = int.from_bytes(data[5:7], byteorder="little", signed=False)
+    return timedelta(days=days, milliseconds=ms)
+
+
+def format_bytes(data: Union[List[int], bytes]) -> str:
+    return ' '.join(list(map(lambda x: hex(x)[2:]
+                             .upper()
+                             .rjust(2, '0'),
+                             data)))
