@@ -3,7 +3,7 @@ from unittest.mock import mock_open, patch
 
 from canopen_monitor.parse import time, eds
 from canopen_monitor.parse.utilities import FailedValidationError
-from . import TEST_EDS
+from tests import TEST_EDS
 
 
 class TestTIME(unittest.TestCase):
